@@ -57,10 +57,11 @@ static
 int
 rbtree_node_compare(PyObject *x, PyObject *y)
 {
+    int gt, lt;
     /* a three way compare that should work with whatever objects support */
-    int gt = PyObject_RichCompareBool(x, y, Py_GT);
+    gt = PyObject_RichCompareBool(x, y, Py_GT);
     if (gt == 1) return 1;
-    int lt = PyObject_RichCompareBool(x, y, Py_LT);
+    lt = PyObject_RichCompareBool(x, y, Py_LT);
     if (lt == 1)  return -1;
     return 0;
 }
@@ -245,11 +246,10 @@ __tree_insert     (rbtree_t *T, PyObject *k, PyObject *v)
                 {
                     // force a value update
                     // XXX: (later) validate this?
-                    Py_XDECREF(x->key);
                     Py_XDECREF(x->value);
-                    x->key = k;
                     x->value = v;
-                    return 0;
+                    Py_INCREF(v);
+                    return NULL;
                 }
             if (rc  < 0) x = x->l;
             else x = x->r;
@@ -284,78 +284,73 @@ __tree_insert     (rbtree_t *T, PyObject *k, PyObject *v)
 }
 
 static
-int
+void
 __rb_insert (rbtree_t *T, PyObject *k, PyObject *v)
 {
-    rbtree_node_t *x = __tree_insert(T, k, v);
-    rbtree_node_t *y;
-
-    if (x ==  NULL) {
-        return 0;
-    }
+    rbtree_node_t *x, *y;
 
     Py_INCREF(k);
     Py_INCREF(v);
 
+    x = __tree_insert(T, k, v);
+    if (x ==  NULL) {
+        return;
+    }
 
-    if (x)
-        {
-            MAKE_RED  (x);
 
-            while ((x != T->root) && (COLOR(x->p) == RED))
-                {
-                    if (x->p == x->p->p->l)
-                        {
-                            y = x->p->p->r;
-                            if (COLOR(y) == RED)
-                                {
-                                    MAKE_BLACK(x->p);
-                                    MAKE_BLACK(y);
-                                    MAKE_RED  (x->p->p);
-                                    x = x->p->p;
-                                }
-                            else
-                                {
-                                    if (x == x->p->r)
-                                        {
-                                            x = x->p;
-                                            __rotate_left(T, x);
-                                        }
-                                    MAKE_BLACK(x->p);
-                                    MAKE_RED  (x->p->p);
-                                    __rotate_right(T, x->p->p);
-                                    break;
-                                }
-                        }
-                    else
-                        {
-                            y = x->p->p->l;
-                            if (COLOR(y) == RED)
-                                {
-                                    MAKE_BLACK(x->p);
-                                    MAKE_BLACK(y);
-                                    MAKE_RED  (x->p->p);
-                                    x = x->p->p;
-                                }
-                            else
-                                {
-                                    if (x == x->p->l)
-                                        {
-                                            x = x->p;
-                                            __rotate_right(T, x);
-                                        }
-                                    MAKE_BLACK(x->p);
-                                    MAKE_RED  (x->p->p);
-                                    __rotate_left(T, x->p->p);
-                                    break;
-                                }
-                        }
-                }
+    MAKE_RED  (x);
 
-            MAKE_BLACK(T->root);
-            return 0;
-        }
-    return 1;
+    while ((x != T->root) && (COLOR(x->p) == RED))
+	{
+	    if (x->p == x->p->p->l)
+		{
+		    y = x->p->p->r;
+		    if (COLOR(y) == RED)
+			{
+			    MAKE_BLACK(x->p);
+			    MAKE_BLACK(y);
+			    MAKE_RED  (x->p->p);
+			    x = x->p->p;
+			}
+		    else
+			{
+			    if (x == x->p->r)
+				{
+				    x = x->p;
+				    __rotate_left(T, x);
+				}
+			    MAKE_BLACK(x->p);
+			    MAKE_RED  (x->p->p);
+			    __rotate_right(T, x->p->p);
+			    break;
+			}
+		}
+	    else
+		{
+		    y = x->p->p->l;
+		    if (COLOR(y) == RED)
+			{
+			    MAKE_BLACK(x->p);
+			    MAKE_BLACK(y);
+			    MAKE_RED  (x->p->p);
+			    x = x->p->p;
+			}
+		    else
+			{
+			    if (x == x->p->l)
+				{
+				    x = x->p;
+				    __rotate_right(T, x);
+				}
+			    MAKE_BLACK(x->p);
+			    MAKE_RED  (x->p->p);
+			    __rotate_left(T, x->p->p);
+			    break;
+			}
+		}
+	}
+
+    MAKE_BLACK(T->root);
 }
 
 static
@@ -481,6 +476,8 @@ __rb_del_node(rbtree_t *T, rbtree_node_t *z)
     DEL(y);
 
     T->ct--;
+    if (remainder == T->nil)
+        return NULL;
     return remainder;
 }
 
@@ -585,10 +582,10 @@ rbtree_free            (rbtree_t *T)
 }
 
 
-int
+void
 rbtree_add             (rbtree_t *T, PyObject *a_ptr, PyObject *value)
 {
-    return __rb_insert(T, a_ptr, value);
+    __rb_insert(T, a_ptr, value);
 }
 
 int
@@ -621,16 +618,18 @@ rbtree_node_get (rbtree_t *T, PyObject *key)
 
 rbtree_node_t *
 tree_min(rbtree_t *T, rbtree_node_t *x) {
+    rbtree_node_t *n;
     if (x == NULL) x = T->root;
-    rbtree_node_t *n = __tree_min(T, x);
+    n = __tree_min(T, x);
     if (n == T->nil) n = NULL;
     return n;
 }
 
 rbtree_node_t *
 tree_max(rbtree_t *T, rbtree_node_t *x) {
+    rbtree_node_t *n;
     if (x == NULL) x = T->root;
-    rbtree_node_t *n = __tree_max(T, x);
+    n = __tree_max(T, x);
     if (n == T->nil) n = NULL;
     return n;
 }
@@ -672,6 +671,9 @@ void rbtree_do_slice(rbtree_t *T, PyObject *start, PyObject *end, PyObject *stri
     else
         en = __tree_search(T, T->root, end, &rc);
 
+    // fix for partial slicing in an empty tree
+    if (!sn) sn = en;
+
     if (stride != Py_None) {
         step = PyInt_AsLong(stride);
         // negative step makes no sense when keys are added via a cmp function
@@ -716,11 +718,7 @@ void rbtree_do_del_slice(rbtree_t *T, PyObject *start, PyObject *end,
         } else {
             cursor = __tree_successor(T, cursor);
         }
-        if (cursor == T->nil) break;
+        if (cursor == NULL || cursor == T->nil) break;
     }
 
 }
-
-
-
-
